# -*- coding: utf-8 -*-


__all__ = ("Map", "WeixinError")


try:
    unicode = unicode
except NameError:
    # python 3
    basestring = (str, bytes)
else:
    # python 2
    bytes = str


class WeixinError(Exception):

    def __init__(self, msg):
        super(WeixinError, self).__init__(msg)


class Map(dict):
    """
    提供字典的dot访问模式
    Example:
    m = Map({'first_name': 'Eduardo'}, last_name='Pool', age=24, sports=['Soccer'])
    """
    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.items():
                    if isinstance(v, dict):
                        v = Map(v)
                    self[k] = v

        if kwargs:
            for k, v in kwargs.items():
                if isinstance(v, dict):
                    v = Map(v)
                self[k] = v

    def __getattr__(self, attr):
        return self[attr]

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __getitem__(self, key):
        if key not in self.__dict__:
            super(Map, self).__setitem__(key, {})
            self.__dict__.update({key: Map()})
        return self.__dict__[key]

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]



import unittest

from weixin.mp import WeixinMP


class TestWeixinMP(unittest.TestCase):
    def setUp(self):
        self.mp = WeixinMP('wxa8d4fa7f869317b7',
                           '4536fbddf0d7c07252320bb30af7808b')

    def test_get_access_token(self):
        self.mp
        self.assertIsNotNone(self.mp.access_token)

    def test_get_jsapi_ticket(self):
        self.mp.get_jsapi_ticket()
        self.assertIsNotNone(self.mp.jsapi_ticket)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

import time
import string
import random
import hashlib
import requests

from .base import Map, WeixinError

try:
    from flask import request
except Exception:
    request = None

from lxml import etree


__all__ = ("WeixinPayError", "WeixinPay")


FAIL = "FAIL"
SUCCESS = "SUCCESS"


class WeixinPayError(WeixinError):

    def __init__(self, msg):
        super(WeixinPayError, self).__init__(msg)


class WeixinPay(object):
    PAY_HOST = "https://api.mch.weixin.qq.com"

    def __init__(self, app_id, mch_id, mch_key, notify_url, key=None, cert=None):
        self.app_id = app_id
        self.mch_id = mch_id
        self.mch_key = mch_key
        self.notify_url = notify_url
        self.key = key
        self.cert = cert
        self.sess = requests.Session()

    @property
    def timestamp(self):
        return str(int(time.time()))

    @property
    def remote_addr(self):
        if request is not None:
            return request.remote_addr
        return ""

    @property
    def nonce_str(self):
        char = string.ascii_letters + string.digits
        return "".join(random.choice(char) for _ in range(32))

    def sign(self, raw):
        raw = [(k, str(raw[k]) if isinstance(raw[k], int) else raw[k])
               for k in sorted(raw.keys())]
        s = "&".join("=".join(kv) for kv in raw if kv[1])
        s += "&key={0}".format(self.mch_key)
        return hashlib.md5(s.encode("utf-8")).hexdigest().upper()

    def check(self, data):
        sign = data.pop("sign")
        return sign == self.sign(data)

    def to_xml(self, raw):
        s = ""
        for k, v in raw.items():
            s += "<{0}>{1}</{0}>".format(k, v)
        s = "<xml>{0}</xml>".format(s)
        return s.encode("utf-8")

    def to_dict(self, content):
        raw = {}
        root = etree.fromstring(content.encode("utf-8"),
                                parser=etree.XMLParser(resolve_entities=False))
        for child in root:
            raw[child.tag] = child.text
        return raw

    def _fetch(self, url, data, use_cert=False, appid=True):
        if appid:
            data.setdefault("appid", self.app_id)
        data.setdefault("mch_id", self.mch_id)
        data.setdefault("nonce_str", self.nonce_str)
        data.setdefault("sign", self.sign(data))

        if use_cert:
            resp = self.sess.post(url, data=self.to_xml(data), cert=(self.cert, self.key))
        else:
            resp = self.sess.post(url, data=self.to_xml(data))
        content = resp.content.decode("utf-8")
        if "return_code" in content:
            data = Map(self.to_dict(content))
            if data.return_code == FAIL:
                raise WeixinPayError(data.return_msg)
            if "result_code" in content and data.result_code == FAIL:
                raise WeixinPayError(data.err_code_des)
            return data
        return content

    def reply(self, msg, ok=True):
        code = SUCCESS if ok else FAIL
        return self.to_xml(dict(return_code=code, return_msg=msg))

    def unified_order(self, **data):
        """
        统一下单
        out_trade_no、body、total_fee、trade_type必填
        app_id, mchid, nonce_str自动填写
        spbill_create_ip 在flask框架下可以自动填写, 非flask框架需要主动传入此参数
        """
        url = self.PAY_HOST + '/pay/unifiedorder'

        # 必填参数
        if "out_trade_no" not in data:
            raise WeixinPayError("缺少统一支付接口必填参数out_trade_no")
        if "body" not in data:
            raise WeixinPayError("缺少统一支付接口必填参数body")
        if "total_fee" not in data:
            raise WeixinPayError("缺少统一支付接口必填参数total_fee")
        if "trade_type" not in data:
            raise WeixinPayError("缺少统一支付接口必填参数trade_type")

        # 关联参数
        if data["trade_type"] == "JSAPI" and "openid" not in data:
            raise WeixinPayError("trade_type为JSAPI时，openid为必填参数")
        if data["trade_type"] == "NATIVE" and "product_id" not in data:
            raise WeixinPayError("trade_type为NATIVE时，product_id为必填参数")
        data.setdefault("notify_url", self.notify_url)
        if "spbill_create_ip" not in data:
            data.setdefault("spbill_create_ip", self.remote_addr)

        raw = self._fetch(url, data)
        return raw

    def jsapi(self, **kwargs):
        """
        生成给JavaScript调用的数据
        详细规则参考 https://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=7_7&index=6
        """
        kwargs.setdefault("trade_type", "JSAPI")
        raw = self.unified_order(**kwargs)
        package = "prepay_id={0}".format(raw["prepay_id"])
        nonce_str = self.nonce_str
        raw = dict(appId=self.app_id, timeStamp=self.timestamp,
                   nonceStr=self.nonce_str, package=package,
                   signType='MD5')
        sign = self.sign(raw)
        raw['paySign'] = sign
        return raw

    order_jsapi = jsapi

    def order_h5(self, **kwargs):
        """
        H5 支付
        详细规则参考 https://pay.weixin.qq.com/wiki/doc/api/H5.php?chapter=9_20&index=1
        """
        kwargs.setdefault('trade_type', 'MWEB')
        return self.unified_order(**kwargs)

    def order_qr(self, **kwargs):
        """
        Native 支付（扫码支付）
        详细规则参考 https://pay.weixin.qq.com/wiki/doc/api/native.php?chapter=9_1
        """
        kwargs.setdefault('trade_type', 'NATIVE')
        return self.unified_order(**kwargs)

    def qrcode_url(self, product_id, shorten=False):
        params = {
            'appid': self.app_id,
            'mch_id': self.mch_id,
            'product_id': product_id,
            'time_stamp': self.timestamp,
            'nonce_str': self.nonce_str,
        }
        params['sign'] = self.sign(params)
        text = ('weixin://wxpay/bizpayurl?sign=%(sign)s&appid=%(appid)s&mch_id=%(mch_id)s'
                '&product_id=%(product_id)s&time_stamp=%(time_stamp)s&nonce_str=%(nonce_str)s') % params
        if shorten:
            return self.qrcode_url_shorten(long_url=urlencode(text))
        return text

    def qrcode_url_shorten(self, **data):
        url = self.PAY_HOST + '/tools/shorturl'
        if 'long_url' not in data:
            raise WechatPayError('缺少转换短链接接口必填参数long_url')
        return self._fetch(url, data)['short_url']

    def order_query(self, **data):
        """
        订单查询
        out_trade_no, transaction_id至少填一个
        appid, mchid, nonce_str不需要填入
        """
        url = self.PAY_HOST + '/pay/orderquery'

        if "out_trade_no" not in data and "transaction_id" not in data:
            raise WeixinPayError("订单查询接口中，out_trade_no、transaction_id至少填一个")

        return self._fetch(url, data)

    def close_order(self, out_trade_no, **data):
        """
        关闭订单
        out_trade_no必填
        appid, mchid, nonce_str不需要填入
        """
        url = self.PAY_HOST + '/pay/closeorder'

        data.setdefault("out_trade_no", out_trade_no)

        return self._fetch(url, data)

    def refund(self, **data):
        """
        申请退款
        out_trade_no、transaction_id至少填一个且
        out_refund_no、total_fee、refund_fee、op_user_id为必填参数
        appid、mchid、nonce_str不需要填入
        """
        url = self.PAY_HOST + '/secapi/pay/refund'
        if not self.key or not self.cert:
            raise WeixinPayError("退款申请接口需要双向证书")
        if "out_trade_no" not in data and "transaction_id" not in data:
            raise WeixinPayError("退款申请接口中，out_trade_no、transaction_id至少填一个")
        if "out_refund_no" not in data:
            raise WeixinPayError("退款申请接口中，缺少必填参数out_refund_no");
        if "total_fee" not in data:
            raise WeixinPayError("退款申请接口中，缺少必填参数total_fee");
        if "refund_fee" not in data:
            raise WeixinPayError("退款申请接口中，缺少必填参数refund_fee");

        return self._fetch(url, data, True)

    def refund_query(self, **data):
        """
        查询退款
        提交退款申请后，通过调用该接口查询退款状态。退款有一定延时，
        用零钱支付的退款20分钟内到账，银行卡支付的退款3个工作日后重新查询退款状态。

        out_refund_no、out_trade_no、transaction_id、refund_id四个参数必填一个
        appid、mchid、nonce_str不需要填入
        """
        url = self.PAY_HOST + '/pay/refundquery'
        if "out_refund_no" not in data and "out_trade_no" not in data \
                and "transaction_id" not in data and "refund_id" not in data:
            raise WeixinPayError("退款查询接口中，out_refund_no、out_trade_no、transaction_id、refund_id四个参数必填一个")

        return self._fetch(url, data)

    def download_bill(self, bill_date, bill_type="ALL", **data):
        """
        下载对账单
        bill_date、bill_type为必填参数
        appid、mchid、nonce_str不需要填入
        """
        url = self.PAY_HOST + '/pay/downloadbill'
        data.setdefault("bill_date", bill_date)
        data.setdefault("bill_type", bill_type)

        if "bill_date" not in data:
            raise WeixinPayError("对账单接口中，缺少必填参数bill_date")

        return self._fetch(url, data)

    def pay_individual(self, **data):
        """
        企业付款到零钱
        """
        url = self.PAY_HOST + '/mmpaymkttransfers/promotion/transfers'
        if not self.key or not self.cert:
            raise WeixinPayError("企业接口需要双向证书")
        if "partner_trade_no" not in data:
            raise WeixinPayError("企业付款接口中, 缺少必要的参数partner_trade_no")
        if "openid" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数openid")
        if "amount" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数amount")
        if "desc" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数desc")
        data.setdefault('check_name', 'NO_CHECK')
        return self._fetch_pay(url, data, True)

    def pay_individual_to_card(self, **data):
        """企业付款到银行卡"""
        url = self.PAY_HOST + '/mmpaysptrans/pay_bank'
        if not self.key or not self.cert:
            raise WeixinPayError("企业接口需要双向证书")
        if "partner_trade_no" not in data:
            raise WeixinPayError("企业付款接口中, 缺少必要的参数partner_trade_no")
        if "enc_bank_no" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数enc_bank_no")
        if "enc_true_name" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数enc_true_name")
        if "bank_code" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数bank_code")
        if "amount" not in data:
            raise WeixinPayError("企业付款接口中，缺少必填参数amount")
        return self._fetch(url, data, True, False)

    def pay_individual_bank_query(self, **data):
        """企业付款到银行卡查询"""
        url = self.PAY_HOST + '/mmpaysptrans/query_bank'
        if not self.key or not self.cert:
            raise WeixinPayError("企业接口需要双向证书'")
        if "partner_trade_no" not in data:
            raise WeixinPayError("企业付款接口中, 缺少必要的参数partner_trade_no")
        return self._fetch(url, data, True, False)

    def pay_individual_query(self, **data):
        """企业付款到零钱查询"""
        url = self.PAY_HOST + '/mmpaymkttransfers/gettransferinfo'
        if not self.key or not self.cert:
            raise WeixinPayError("企业接口需要双向证书'")
        if "partner_trade_no" not in data:
            raise WeixinPayError("企业付款接口中, 缺少必要的参数partner_trade_no")
        return self._fetch(url, data, True)

    def _fetch_pay(self, url, data, use_cert=False):
        data.setdefault("mch_appid", self.app_id)
        data.setdefault("mchid", self.mch_id)
        data.setdefault("nonce_str", self.nonce_str)
        data.setdefault("sign", self.sign(data))
        if use_cert:
            resp = self.sess.post(url, data=self.to_xml(data), cert=(self.cert, self.key))
        else:
            resp = self.sess.post(url, data=self.to_xml(data))
        content = resp.content.decode("utf-8")
        if "return_code" in content:
            data = Map(self.to_dict(content))
            if data.return_code == FAIL:
                raise WeixinPayError(data.return_msg)
            if "result_code" in content and data.result_code == FAIL:
                raise WeixinPayError(data.err_code_des)
            return data
        return content



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

import hashlib
import time
from datetime import datetime

from weixin.base import WeixinError

try:
    from .base import basestring
except ImportError:
    pass

try:
    from flask import Response, request
except ImportError:
    request, Response = None, None

try:
    from django.http import (HttpResponse, HttpResponseForbidden,
                             HttpResponseNotAllowed)
except Exception:
    HttpResponse, HttpResponseForbidden, HttpResponseNotAllowed = None, None, None

from lxml import etree

__all__ = ("WeixinMsgError", "WeixinMsg")


class WeixinMsgError(WeixinError):

    def __init__(self, msg):
        super(WeixinMsgError, self).__init__(msg)


class WeixinMsg(object):

    def __init__(self, token, sender=None, expires_in=0):
        self.token = token
        self.sender = sender
        self.expires_in = expires_in
        self._registry = dict()

    def validate(self, signature, timestamp, nonce):
        print("signature: %s, timestamp: %s, nonce: %s" %
              (signature, timestamp, nonce))
        if not self.token:
            raise WeixinMsgError("weixin token is missing")

        if self.expires_in:
            try:
                timestamp = int(timestamp)
            except ValueError:
                return False
            delta = time.time() - timestamp
            if delta < 0 or delta > self.expires_in:
                return False
        values = [self.token, str(timestamp), str(nonce)]
        s = ''.join(sorted(values))
        hsh = hashlib.sha1(s.encode("utf-8")).hexdigest()
        return signature == hsh

    def parse(self, content):
        raw = {}
        root = etree.fromstring(content,
                                parser=etree.XMLParser(resolve_entities=False))
        for child in root:
            raw[child.tag] = child.text

        formatted = self.format(raw)
        msg_type = formatted['type']
        msg_parser = getattr(self, 'parse_{0}'.format(msg_type), None)
        if callable(msg_parser):
            parsed = msg_parser(raw)
        else:
            parsed = self.parse_invalid_type(raw)

        formatted.update(parsed)
        return formatted

    def format(self, kwargs):
        timestamp = int(kwargs['CreateTime'])
        return {
            'id': kwargs.get('MsgId'),
            'timestamp': timestamp,
            'receiver': kwargs['ToUserName'],
            'sender': kwargs['FromUserName'],
            'type': kwargs['MsgType'],
            'time': datetime.fromtimestamp(timestamp),
        }

    def parse_text(self, raw):
        return {'content': raw['Content']}

    def parse_image(self, raw):
        return {'picurl': raw['PicUrl']}

    def parse_location(self, raw):
        return {
            'location_x': raw['Location_X'],
            'location_y': raw['Location_Y'],
            'scale': int(raw.get('Scale', 0)),
            'label': raw['Label'],
        }

    def parse_link(self, raw):
        return {
            'title': raw['Title'],
            'description': raw['Description'],
            'url': raw['url'],
        }

    def parse_voice(self, raw):
        return {
            'media_id': raw['MediaId'],
            'format': raw['Format'],
            'recognition': raw['Recognition'],
        }

    def parse_video(self, raw):
        return {
            'media_id': raw['MediaId'],
            'thumb_media_id': raw['ThumbMediaId'],
        }

    def parse_shortvideo(self, raw):
        return {
            'media_id': raw['MediaId'],
            'thumb_media_id': raw['ThumbMediaId'],
        }

    def parse_event(self, raw):
        return {
            'event': raw.get('Event'),
            'event_key': raw.get('EventKey'),
            'ticket': raw.get('Ticket'),
            'latitude': raw.get('Latitude'),
            'longitude': raw.get('Longitude'),
            'precision': raw.get('Precision'),
            'status': raw.get('status')
        }

    def parse_invalid_type(self, raw):
        return {}

    def reply(self, username=None, type='text', sender=None, **kwargs):
        print("reply: username: %s, type: %s, sender: %s, kwargs: %s" %
              (username, type, sender, kwargs))

        if not username:
            raise RuntimeError("username is missing")
        sender = sender or self.sender
        if not sender:
            raise RuntimeError('WEIXIN_SENDER or sender argument is missing')

        if type == 'text':
            content = kwargs.get('content', '')
            return text_reply(username, sender, content)

        if type == 'music':
            values = {}
            for k in ('title', 'description', 'music_url', 'hq_music_url'):
                values[k] = kwargs[k]
            return music_reply(username, sender, **values)

        if type == 'news':
            items = kwargs['articles']
            return news_reply(username, sender, *items)

        if type == 'customer_service':
            service_account = kwargs['service_account']
            return transfer_customer_service_reply(username, sender,
                                                   service_account)

        if type == 'image':
            media_id = kwargs.get('media_id')
            return image_reply(username, sender, media_id)

        if type == 'voice':
            media_id = kwargs.get('media_id')
            return voice_reply(username, sender, media_id)

        if type == 'video':
            values = {}
            for k in ('media_id', 'title', 'description'):
                values[k] = kwargs[k]
            return video_reply(username, sender, **values)

    def register(self, type, key=None, func=None):
        if func:
            key = '*' if not key else key
            self._registry.setdefault(type, dict())[key] = func
            return func
        return self.__call__(type, key)

    def __call__(self, type, key):
        def wrapper(func):
            self.register(type, key, func)
            return func
        return wrapper

    @property
    def all(self):
        return self.register('*')

    def text(self, key='*'):
        return self.register('text', key)

    def __getattr__(self, key):
        key = key.lower()
        if key in ['image', 'video', 'voice', 'shortvideo', 'location', 'link', 'event']:
            return self.register(key)
        if key in ['subscribe', 'unsubscribe', 'location', 'click', 'view', 'scan',
                   'scancode_push', 'scancode_waitmsg', 'pic_sysphoto',
                   'pic_photo_or_album', 'pic_weixin', 'location_select',
                   'qualification_verify_success', 'qualification_verify_fail', 'naming_verify_success',
                   'naming_verify_fail', 'annual_renew', 'verify_expired',
                   'card_pass_check', 'user_get_card', 'user_del_card', 'user_consume_card',
                   'user_pay_from_pay_cell', 'user_view_card', 'user_enter_session_from_card',
                   'card_sku_remind']:
            return self.register('event', key)
        raise AttributeError('invalid attribute "' + key + '"')

    def django_view_func(self):

        def run(request):
            if HttpResponse is None:
                raise RuntimeError('django_view_func need Django be installed')
            signature = request.GET.get('signature')

            timestamp = request.GET.get('timestamp')
            nonce = request.GET.get('nonce')
            if not self.validate(signature, timestamp, nonce):
                return HttpResponseForbidden('signature failed')
            if request.method == 'GET':
                echostr = request.GET.get('echostr', '')
                return HttpResponse(echostr)
            elif request.method == "POST":
                try:
                    ret = self.parse(request.body)
                except ValueError:
                    return HttpResponseForbidden('invalid')

                func = None
                type = ret['type']
                _registry = self._registry.get(type, dict())
                if type == 'text':
                    if ret['content'] in _registry:
                        func = _registry[ret['content']]
                elif type == 'event':
                    if ret['event'].lower() in _registry:
                        func = _registry[ret['event'].lower()]

                if func is None and '*' in _registry:
                    func = _registry['*']
                if func is None and '*' in self._registry:
                    func = self._registry.get('*', dict()).get('*')

                text = ''
                if func is None:
                    text = 'failed'

                if callable(func):
                    text = func(**ret)

                content = ''
                if isinstance(text, basestring):
                    if text:
                        content = self.reply(
                            username=ret['sender'],
                            sender=ret['receiver'],
                            content=text,
                        )
                elif isinstance(text, dict):
                    text.setdefault('username', ret['sender'])
                    text.setdefault('sender', ret['receiver'])
                    content = self.reply(**text)

                return HttpResponse(content, content_type='text/xml; charset=utf-8')
            return HttpResponseNotAllowed(['GET', 'POST'])
        return run

    def view_func(self):
        if request is None:
            raise RuntimeError('view_func need Flask be installed')

        signature = request.args.get('signature')
        timestamp = request.args.get('timestamp')
        nonce = request.args.get('nonce')
        if not self.validate(signature, timestamp, nonce):
            return 'signature failed', 400
        if request.method == 'GET':
            echostr = request.args.get('echostr', '')
            return echostr

        try:
            ret = self.parse(request.data)
        except ValueError:
            return 'invalid', 400

        func = None
        type = ret['type']
        _registry = self._registry.get(type, dict())
        if type == 'text':
            if ret['content'] in _registry:
                func = _registry[ret['content']]
        elif type == 'event':
            if ret['event'].lower() in _registry:
                func = _registry[ret['event'].lower()]

        if func is None and '*' in _registry:
            func = _registry['*']
        if func is None and '*' in self._registry:
            func = self._registry.get('*', dict()).get('*')

        text = ''
        if func is None:
            text = 'failed'

        if callable(func):
            text = func(**ret)

        content = ''
        if isinstance(text, basestring):
            if text:
                content = self.reply(
                    username=ret['sender'],
                    sender=ret['receiver'],
                    content=text,
                )
        elif isinstance(text, dict):
            text.setdefault('username', ret['sender'])
            text.setdefault('sender', ret['receiver'])
            content = self.reply(**text)

        return Response(content, content_type='text/xml; charset=utf-8')

    view_func.methods = ['GET', 'POST']


def text_reply(username, sender, content):
    print("username: %s, sender: %s, content: %s" %
          (username, sender, content))
    shared = _shared_reply(username, sender, 'text')
    template = '<xml>%s<Content><![CDATA[%s]]></Content></xml>'
    return template % (shared, content)


def music_reply(username, sender, **kwargs):
    print("username: %s, sender: %s, kwargs: %s" % (username, sender, kwargs))
    kwargs['shared'] = _shared_reply(username, sender, 'music')

    template = (
        '<xml>'
        '%(shared)s'
        '<Music>'
        '<Title><![CDATA[%(title)s]]></Title>'
        '<Description><![CDATA[%(description)s]]></Description>'
        '<MusicUrl><![CDATA[%(music_url)s]]></MusicUrl>'
        '<HQMusicUrl><![CDATA[%(hq_music_url)s]]></HQMusicUrl>'
        '</Music>'
        '</xml>'
    )
    return template % kwargs


def news_reply(username, sender, *items):
    print("username: %s, sender: %s, items: %s" % (username, sender, items))
    item_template = (
        '<item>'
        '<Title><![CDATA[%(title)s]]></Title>'
        '<Description><![CDATA[%(description)s]]></Description>'
        '<PicUrl><![CDATA[%(picurl)s]]></PicUrl>'
        '<Url><![CDATA[%(url)s]]></Url>'
        '</item>'
    )
    articles = [item_template % o for o in items]

    template = (
        '<xml>'
        '%(shared)s'
        '<ArticleCount>%(count)d</ArticleCount>'
        '<Articles>%(articles)s</Articles>'
        '</xml>'
    )
    dct = {
        'shared': _shared_reply(username, sender, 'news'),
        'count': len(items),
        'articles': ''.join(articles)
    }
    return template % dct


def transfer_customer_service_reply(username, sender, service_account):
    template = (
        '<xml>%(shared)s'
        '%(transfer_info)s</xml>')
    transfer_info = ''
    if service_account:
        transfer_info = (
            '<TransInfo>'
            '<KfAccount>![CDATA[%s]]</KfAccount>'
            '</TransInfo>') % service_account

    dct = {
        'shared': _shared_reply(username, sender,
                                type='transfer_customer_service'),
        'transfer_info': transfer_info
    }
    return template % dct


def image_reply(username, sender, media_id):
    print("username: %s, sender: %s, media_id: %s" %
          (username, sender, media_id))
    shared = _shared_reply(username, sender, 'image')
    template = '<xml>%s<Image><MediaId><![CDATA[%s]]></MediaId></Image></xml>'
    return template % (shared, media_id)


def voice_reply(username, sender, media_id):
    print("username: %s, sender: %s, media_id: %s" %
          (username, sender, media_id))
    shared = _shared_reply(username, sender, 'voice')
    template = '<xml>%s<Voice><MediaId><![CDATA[%s]]></MediaId></Voice></xml>'
    return template % (shared, media_id)


def video_reply(username, sender, **kwargs):
    print("username: %s, sender: %s, kwargs: %s" % (username, sender, kwargs))
    kwargs['shared'] = _shared_reply(username, sender, 'video')

    template = (
        '<xml>'
        '%(shared)s'
        '<Video>'
        '<MediaId><![CDATA[%(media_id)s]]></MediaId>'
        '<Title><![CDATA[%(title)s]]></Title>'
        '<Description><![CDATA[%(description)s]]></Description>'
        '</Video>'
        '</xml>'
    )
    return template % kwargs


def _shared_reply(username, sender, type):
    print("username: %s, sender: %s, type: %s" % (username, sender, type))
    dct = {
        'username': username,
        'sender': sender,
        'type': type,
        'timestamp': int(time.time()),
    }
    template = (
        '<ToUserName><![CDATA[%(username)s]]></ToUserName>'
        '<FromUserName><![CDATA[%(sender)s]]></FromUserName>'
        '<CreateTime>%(timestamp)d</CreateTime>'
        '<MsgType><![CDATA[%(type)s]]></MsgType>'
    )
    return template % dct



# -*- coding: utf-8 -*-


from collections import namedtuple

from weixin.login import WeixinLogin
from weixin.mp import WeixinMP
from weixin.msg import WeixinMsg
from weixin.pay import WeixinPay

__all__ = ("Weixin",)
__author__ = "Weicheng Zou <zwczou@gmail.com>"
__version__ = "0.5.9"


StandaloneApplication = namedtuple("StandaloneApplication", ["config"])


class Weixin(WeixinLogin, WeixinPay, WeixinMP, WeixinMsg):
    """
    微信SDK

    :param app 如果非flask，传入字典配置，如果是flask直接传入app实例
    """

    def __init__(self, app=None):
        if app is not None:
            if isinstance(app, dict):
                app = StandaloneApplication(config=app)
            self.init_app(app)
            self.app = app

    def init_app(self, app):
        if isinstance(app, dict):
            app = StandaloneApplication(config=app)

        token = app.config.get("WEIXIN_TOKEN")
        sender = app.config.get('WEIXIN_SENDER', None)
        expires_in = app.config.get('WEIXIN_EXPIRES_IN', 0)
        mch_id = app.config.get("WEIXIN_MCH_ID")
        mch_key = app.config.get("WEIXIN_MCH_KEY")
        notify_url = app.config.get("WEIXIN_NOTIFY_URL")
        mch_key_file = app.config.get("WEIXIN_MCH_KEY_FILE")
        mch_cert_file = app.config.get("WEIXIN_MCH_CERT_FILE")
        app_id = app.config.get("WEIXIN_APP_ID")
        app_secret = app.config.get("WEIXIN_APP_SECRET")
        if token:
            WeixinMsg.__init__(self, token, sender, expires_in)
        if app_id and mch_id and mch_key and notify_url:
            WeixinPay.__init__(self, app_id, mch_id, mch_key,
                               notify_url, mch_key_file, mch_cert_file)
        if app_id and app_secret:
            WeixinLogin.__init__(self, app_id, app_secret)
            WeixinMP.__init__(self, app_id, app_secret)

        # 兼容老版本
        if app_id and mch_id and mch_key and notify_url:
            self.pay = WeixinPay(app_id, mch_id, mch_key,
                                 notify_url, mch_key_file, mch_cert_file)
        if token:
            self.msg = WeixinMsg(token, sender, expires_in)
        if app_id and app_secret:
            self.login = WeixinLogin(app_id, app_secret)
            self.mp = WeixinMP(app_id, app_secret)



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

import json
import requests

from .base import Map, WeixinError


__all__ = ("WeixinLoginError", "WeixinLogin")


class WeixinLoginError(WeixinError):

    def __init__(self, msg):
        super(WeixinLoginError, self).__init__(msg)


class WeixinLogin(object):

    def __init__(self, app_id, app_secret):
        self.sess = requests.Session()
        self.app_id = app_id
        self.app_secret = app_secret

    def _get(self, url, params):
        resp = self.sess.get(url, params=params)
        data = Map(json.loads(resp.content.decode("utf-8")))
        if data.errcode:
            msg = "%(errcode)d %(errmsg)s" % data
            raise WeixinLoginError(msg)
        return data

    def authorize(self, redirect_uri, scope="snsapi_base", state=None):
        """
        生成微信认证地址并且跳转

        :param redirect_uri: 跳转地址
        :param scope: 微信认证方式，有`snsapi_base`跟`snsapi_userinfo`两种
        :param state: 认证成功后会原样带上此字段
        """
        url = "https://open.weixin.qq.com/connect/oauth2/authorize"
        assert scope in ["snsapi_base", "snsapi_userinfo"]
        data = dict()
        data.setdefault("appid", self.app_id)
        data.setdefault("redirect_uri", redirect_uri)
        data.setdefault("response_type", "code")
        data.setdefault("scope", scope)
        if state:
            data.setdefault("state", state)
        data = [(k, data[k]) for k in sorted(data.keys()) if data[k]]
        s = "&".join("=".join(kv) for kv in data if kv[1])
        return "{0}?{1}#wechat_redirect".format(url, s)

    def access_token(self, code):
        """
        获取令牌
        """
        url = "https://api.weixin.qq.com/sns/oauth2/access_token"
        args = dict()
        args.setdefault("appid", self.app_id)
        args.setdefault("secret", self.app_secret)
        args.setdefault("code", code)
        args.setdefault("grant_type", "authorization_code")

        return self._get(url, args)

    def auth(self, access_token, openid):
        """
        检验授权凭证

        :param access_token: 授权凭证
        :param openid: 唯一id
        """
        url = "https://api.weixin.qq.com/sns/auth"
        args = dict()
        args.setdefault("access_token", access_token)
        args.setdefault("openid", openid)

        return self._get(url, args)

    def refresh_token(self, refresh_token):
        """
        重新获取access_token

        :param refresh_token: 刷新令牌
        """
        url = "https://api.weixin.qq.com/sns/oauth2/refresh_token"
        args = dict()
        args.setdefault("appid", self.app_id)
        args.setdefault("grant_type", "refresh_token")
        args.setdefault("refresh_token", refresh_token)

        return self._get(url, args)

    def userinfo(self, access_token, openid):
        """
        获取用户信息

        :param access_token: 令牌
        :param openid: 用户id，每个应用内唯一
        """
        url = "https://api.weixin.qq.com/sns/userinfo"
        args = dict()
        args.setdefault("access_token", access_token)
        args.setdefault("openid", openid)
        args.setdefault("lang", "zh_CN")

        return self._get(url, args)

    def user_info(self, access_token, openid):
        """
        获取用户信息

        兼容老版本0.3.0,与WeixinMP的user_info冲突
        """
        return self.userinfo(access_token, openid)

    def jscode2session(self, js_code):
        """
        小程序获取 session_key 和 openid
        """
        url = "https://api.weixin.qq.com/sns/jscode2session"
        args = dict()
        args.setdefault("appid", self.app_id)
        args.setdefault("secret", self.app_secret)
        args.setdefault("js_code", js_code)
        args.setdefault("grant_type", "authorization_code")
        return self._get(url, args)



import unittest

from weixin import WeixinMsg


class TestWeixinMsg(unittest.TestCase):
    def setUp(self):
        self.msg = WeixinMsg('TwigC0des')

    def test_text_reply(self):
        self.assertRegex(self.msg.reply(
            username='test',
            type='text',
            sender='sender',
            content='Hello, world!'
        ),
            "<xml><ToUserName><!\[CDATA\[test]]></ToUserName><FromUserName><!\[CDATA\[sender]]></FromUserName><CreateTime>[0-9]+</CreateTime><MsgType><!\[CDATA\[text]]></MsgType><Content><!\[CDATA\[Hello, world!]]></Content></xml>")

    def test_news_reply(self):
        self.assertRegex(self.msg.reply(
            username='test',
            type='news',
            sender='sender',
            articles=[dict(
                title='Hello, world!',
                description='Hello, world!',
                picurl='http://www.baidu.com/img/bdlogo.gif',
                url='http://www.baidu.com'
            )]
        ),
            "<xml><ToUserName><!\[CDATA\[test]]></ToUserName><FromUserName><!\[CDATA\[sender]]></FromUserName><CreateTime>[0-9]+</CreateTime><MsgType><!\[CDATA\[news]]></MsgType><ArticleCount>1</ArticleCount><Articles><item><Title><!\[CDATA\[Hello, world!]]></Title><Description><!\[CDATA\[Hello, world!]]></Description><PicUrl><!\[CDATA\[http://www\.baidu\.com/img/bdlogo\.gif]]></PicUrl><Url><!\[CDATA\[http://www\.baidu\.com]]></Url></item></Articles></xml>")


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-


from __future__ import unicode_literals

import hashlib
import json
import os
import random
import string
import time

import requests

from weixin.base import Map, WeixinError

__all__ = ("WeixinMPError", "WeixinMP")


DEFAULT_DIR = os.getenv("HOME", os.getcwd())


class WeixinMPError(WeixinError):

    def __init__(self, msg):
        super(WeixinMPError, self).__init__(msg)


class WeixinMP(object):
    """
    微信公众号相关接口

    当需要全局使用access token可以选择继承WeixinMP实现access_token

        class WeixinMPSub(object):

            def __init__(self, app_id, app_secret):
                WeixinMP.__init__(app_id, app_secret)

            @property
            def access_token(self):
                return requests.get("http://example.com").content


        mp = WeixinMPSub("app_id", "app_secret")

    也可以选择传入jt_callback

        def get_access_token(mp):
            return requests.get("http://example.com").content

        WeixinMP("app_id", "app_secret", ac_callback=get_access_token)
    """

    api_uri = "https://api.weixin.qq.com"

    def __init__(self, app_id, app_secret, ac_path=None, jt_path=None, ac_callback=None, jt_callback=None):
        """
        :param :app_id 微信app id
        :param :app_secret 微信app secret
        :param :ac_path access token 保存路径
        :param :jt_path js ticket 保存路径
        :param :ac_callback ac_callback
        :param :jt_callback jt_callback
        """
        self.app_id = app_id
        self.app_secret = app_secret
        self.session = requests.Session()
        if ac_path is None:
            ac_path = os.path.join(DEFAULT_DIR, ".access_token")
        if jt_path is None:
            jt_path = os.path.join(DEFAULT_DIR, ".jsapi_ticket")
        self.ac_path = ac_path
        self.jt_path = jt_path
        self.ac_callback = ac_callback
        self.jt_callback = jt_callback

    def fetch(self, method, url, params=None, data=None, headers=None):
        req = requests.Request(method, url, params=params,
                               data=data, headers=headers)
        prepped = req.prepare()
        resp = self.session.send(prepped, timeout=20)
        data = Map(resp.json())
        if data.errcode:
            msg = "%(errcode)d %(errmsg)s" % data
            raise WeixinMPError(msg)
        return data

    def get(self, path, params=None, token=True, prefix="/cgi-bin"):
        if path.startswith("http"):
            url = path
        else:
            url = "{0}{1}{2}".format(self.api_uri, prefix, path)
        params = {} if not params else params
        token and params.setdefault("access_token", self.access_token)
        return self.fetch("GET", url, params)

    def post(self, path, data, prefix="/cgi-bin", json_encode=True, token=True):
        if path.startswith("http"):
            url = path
        else:
            url = "{0}{1}{2}".format(self.api_uri, prefix, path)
        url = "{0}{1}{2}".format(self.api_uri, prefix, path)
        params = {}
        token and params.setdefault("access_token", self.access_token)
        headers = {}
        if json_encode:
            # data = json.dumps(data, ensure_ascii=False)
            data = json.dumps(data)
            headers["Content-Type"] = "application/json;charset=UTF-8"
        # print url, params, headers, data
        return self.fetch("POST", url, params=params, data=data, headers=headers)

    @property
    def access_token(self):
        """
        获取服务端凭证

        当多台服务器需要共用access_token的时候
        如果不想自己继承实现access_token, 可以传入ac_callback()
        接收一个WeixinMP对象作为参数
        """
        if self.ac_callback and callable(self.ac_callback):
            return self.ac_callback(self)

        timestamp = time.time()
        if not os.path.exists(self.ac_path) or \
                int(os.path.getmtime(self.ac_path)) < timestamp:
            params = dict()
            params.setdefault("grant_type", "client_credential")
            params.setdefault("appid", self.app_id)
            params.setdefault("secret", self.app_secret)
            data = self.get("/token", params, token=False)
            with open(self.ac_path, 'wb') as fp:
                fp.write(data.access_token.encode("utf-8"))
            os.utime(self.ac_path, (timestamp, timestamp + data.expires_in - 600))
        return open(self.ac_path).read().strip()

    @property
    def jsapi_ticket(self):
        """
        获取jsapi ticket

        当多台服务器需要共用js_ticket的时候
        如果不想自己继承实现js_ticket, 可以传入jt_callback()
        接收一个WeixinMP对象作为参数
        """
        if self.jt_callback and callable(self.jt_callback):
            return self.jt_callback(self)

        timestamp = time.time()
        if not os.path.exists(self.jt_path) or \
                int(os.path.getmtime(self.jt_path)) < timestamp:
            params = dict()
            params.setdefault("type", "jsapi")
            data = self.get("/ticket/getticket", params)
            with open(self.jt_path, 'wb') as fp:
                fp.write(data.ticket.encode("utf-8"))
            os.utime(self.jt_path, (timestamp, timestamp + data.expires_in - 600))
        return open(self.jt_path).read()

    @property
    def nonce_str(self):
        char = string.ascii_letters + string.digits
        return "".join(random.choice(char) for _ in range(32))

    def jsapi_sign(self, **kwargs):
        """
        生成签名给js使用
        """
        timestamp = str(int(time.time()))
        nonce_str = self.nonce_str
        kwargs.setdefault("jsapi_ticket", self.jsapi_ticket)
        kwargs.setdefault("timestamp", timestamp)
        kwargs.setdefault("noncestr", nonce_str)
        raw = [(k, kwargs[k]) for k in sorted(kwargs.keys())]
        s = "&".join("=".join(kv) for kv in raw if kv[1])
        sign = hashlib.sha1(s.encode("utf-8")).hexdigest().lower()
        return Map(sign=sign, timestamp=timestamp, noncestr=nonce_str, appId=self.app_id)

    def groups_create(self, name):
        """
        创建分组

        :param name: 分组名
        """
        data = dict(group=dict(name=name))
        return self.post("/groups/create", data)

    def groups_get(self):
        """
        获取所有分组
        """
        return self.get("/groups/get")

    def groups_getid(self, openid):
        """
        查询用户所在分组

        :param openid: 用户id
        """
        data = dict(openid=openid)
        return self.post("/groups/getid", data)

    def groups_update(self, id, name):
        """
        修改分组名

        :param id: 分组id
        :param name: 分组名
        """
        data = dict(group=dict(id=id, name=name))
        return self.post("/groups/update", data)

    def groups_members_update(self, to_groupid, openid):
        """
        移动用户分组

        :param to_groupid: 分组id
        :param openid: 用户唯一标识符
        """
        data = dict(openid=openid, to_groupid=to_groupid)
        return self.post("/groups/members/update", data)

    def groups_members_batchupdate(self, to_groupid, *openid):
        """
        批量移动用户分组

        :param to_groupid: 分组id
        :param openid: 用户唯一标示列表
        """
        data = dict(openid_list=openid, to_groupid=to_groupid)
        return self.post("/groups/members/batchupdate", data)

    def groups_delete(self, id):
        """
        删除组

        :param id: 分组的id
        """
        data = dict(group=dict(id=id))
        return self.post("/groups/delete", data)

    def user_info_updateremark(self, openid, remark):
        """
        设置备注名

        :param openid: 用户唯一标识符
        :param remark: 备注
        """
        data = dict(openid=openid, remark=remark)
        return self.post("/user/info/updateremark", data)

    def user_info(self, openid):
        """
        获取用户信息
        包含subscribe字段，可以用来判断用户是否关注公众号

        :param openid: 用户id
        """
        args = dict(openid=openid, lang="zh_CN")
        return self.get("/user/info", args)

    def user_info_batchget(self, *openids):
        """
        批量获取用户信息
        """
        user_list = []
        for openid in openids:
            user_list.append(dict(openid=openid, lang="zh_CN"))
        data = dict(user_list=user_list)
        return self.post("/user/info/batchget", data)

    def user_get(self, next_openid=None):
        """
        获取公众号关注列表
        一次最多返回1000个

        :param next_openid: 第一个拉取的openid,不填默认从头开始
        """
        args = dict()
        if next_openid:
            args.setdefault("next_openid", next_openid)
        return self.get("/user/get", args)

    def menu_create(self, data):
        data = dict(button=data)
        return self.post("/menu/create", data)

    def menu_get(self):
        return self.get("/menu/get")

    def menu_delete(self):
        return self.get("/menu/delete")

    def get_current_selfmenu_info(self):
        return self.get("/get_current_selfmenu_info")

    def shorturl(self, long_url):
        """
        长链接转为短链接

        :param long_url: 长链接
        """
        data = dict(action="long2short", long_url=long_url)
        return self.post("/shorturl", data)

    def qrcode_create(self, scene_id, expires=30):
        """
        创建qrcode
        """
        data = dict(
            action_name="QR_SCENE", expire_seconds=expires,
            action_info=dict(scene=dict(scene_id=scene_id)),
        )
        return self.post("/qrcode/create", data)

    def qrcode_create_limit(self, scene):
        """
        创建qrcode限制方式
        """
        data = dict()
        if isinstance(scene, int):
            data["action_name"] = "QR_LIMIT_SCENE"
            data["action_info"] = dict(scene=dict(
                scene_id=scene,
            ))
        elif isinstance(scene, str):
            data["action_name"] = "QR_LIMIT_STR_SCENE"
            data["action_info"] = dict(scene=dict(
                scene_str=scene,
            ))
        else:
            raise ValueError("invalid type")
        return self.post("/qrcode/create", data)

    def qrcode_show(self, ticket):
        """
        显示qrcode
        """
        url = "https://mp.weixin.qq.com/cgi-bin/showqrcode"
        return self.get(url, dict(ticket=ticket))

    def shop_list(self, pageindex=1, pagesize=10):
        """
        门店列表
        """
        data = dict(pageindex=pageindex, pagesize=pagesize)
        return self.post("/shop/list", data, prefix="/bizwifi")

    def shop_get(self, shop_id):
        """
        查询门店Wi-Fi信息
        """
        return self.post("/shop/get", dict(shop_id=shop_id), prefix="/bizwifi")

    def shop_update(self, shop_id, old_ssid, ssid, password=None):
        """
        修改门店网络信息
        """
        data = dict(shop_id=shop_id, old_ssid=old_ssid, ssid=ssid)
        if password:
            data.update(dict(password=password))
        return self.post("/shop/update", data, prefix="/bizwifi")

    def shop_clean(self, shop_id):
        """
        通过此接口清空门店的网络配置及所有设备，恢复空门店状态
        """
        return self.post("/shop/clean", dict(shop_id=shop_id), prefix="/bizwifi")

    def apportal_register(self, shop_id, ssid, reset):
        """
        添加portal型设备
        """
        data = dict(shop_id=shop_id, ssid=ssid, reset=reset)
        return self.post("/apportal/register", data)

    def device_list(self, shop_id=None, pageindex=1, pagesize=10, prefix="/bizwifi"):
        """
        查询设备
        """
        data = dict(pageindex=pageindex, pagesize=pagesize)
        if shop_id:
            data.update(dict(shop_id=shop_id))
        return self.post("/device/list", data, prefix="/bizwifi")

    def device_delete(self, bssid):
        """
        删除设备
        """
        return self.post("/device/delete", dict(bssid=bssid), prefix="/bizwifi")

    def qrcode_get(self, shop_id, ssid, img_id):
        """
        获取物料二维码
        """
        data = dict(shop_id=shop_id, ssid=ssid, img_id=img_id)
        return self.post("/qrcode/get", data, prefix="/bizwifi")

    def get_all_private_template(self):
        """
        获取所有私有模板列表
        """
        return self.get("/template/get_all_private_template")

    def del_private_template(self, template_id):
        """
        删除私有模板
        """
        return self.post("/template/del_private_template", dict(template_id=template_id))

    def template_send(self, template_id, touser, data, url=None, miniprogram=None, **kwargs):
        """
        发送模板消息

        :params template_id: 模板id
        :params touser: openid
        :params data: 模板消息对应的内容跟颜色
        :params url: 跳转地址
        :params miniprogram: 小程序跳转相关
        """
        kwargs.setdefault("template_id", template_id)
        kwargs.setdefault("touser", touser)
        kwargs.setdefault("data", data)
        url and kwargs.setdefault("url", url)
        miniprogram and kwargs.setdefault("miniprogram", miniprogram)
        # print kwargs
        return self.post("/message/template/send", kwargs)



# -*- coding: utf-8 -*-


import ast
import os
import re

from setuptools import find_packages, setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('weixin/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')
    ).group(1)))


def fread(fname):
    filepath = os.path.join(os.path.dirname(__file__), fname)
    if os.path.exists(filepath):
        with open(filepath, encoding='utf8') as f:
            return f.read()


setup(
    name='wechat-python',
    description='Wechat for Python',
    long_description=fread('docs/quickstart.rst'),
    long_description_content_type='text/x-rst',
    license='BSD',
    packages=find_packages(),
    version=version,
    author='zwczou',
    author_email='zwczou@gmail.com',
    url='https://github.com/wpcfan/weixin-python',
    keywords=['weixin', 'weixin pay', 'weixin login', 'weixin mp', 'weixin python',
              'wechat', 'wechat pay', 'wechat login', 'wechat mp', 'wechat python'],
    zip_safe=False,
    platforms='any',
    install_requires=[
        "requests",
        "lxml"
    ],
    classifiers=[],
)
